package org.eclipse.gmt.gems.model.intelligence;
/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and
 * the accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/

/**
 * This is a key interface for implementing a new type
 * of constraint language. Implementations of this interface
 * are responsible for taking a source and target object of
 * a relationship and determining if a constraint in the
 * new constraint language holds true across the source
 * and target objects. Implementations of this interface
 * should be registered with the RefreshKB via a call to
 * <code>
 * ConstraintEvaluator eval = new MyConstraintEvaluator();
 * RefreshKB kb = ...;
 * kb.getConstraintEvaluators().add(eval);
 * </code>
 */
public interface ConstraintEvaluator {	
   
	/**
	 * This method returns true if the specified
	 * Constraint holds for the given source and
	 * target object.
	 * 
	 * @param src the source of the relationship
	 * @param trg the target of the relationship
	 * @param constraint the constraint to evaluate
	 * @return true if the constraint holds
	 */
   public boolean evaluate(Object src, Object trg, Constraint constraint);
}
