/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/
package org.eclipse.gmt.gems.model.intelligence;

/**
 * This interface is for classes that
 * can construct instances of Constraint
 * objects from raw constraint data.
 * Generally, this interface should be
 * implemented by classes that contribute
 * new types of Constraints and ConstraintEvaluators.
 * Each ConstraintFactory should be registered
 * with KBConstraintFactory so that they
 * are accessible to the outside world.
 * 
 * @author Jules White
 *
 */
public interface ConstraintFactory {
	
	public static final String EXTENSION_POINT = "org.eclipse.gmt.gems.model.intelligence.core.constraintfactory";
	
	/**
	 * This method constructs a constraint
	 * object using the specified data. What
	 * the data includes is implementation
	 * specific. Generally, it makes sense
	 * to allow this method to accept
	 * String representations of constraints
	 * so that they can be fed in from
	 * configuration files.
	 * 
	 * @param data to construct the constraint with
	 * @return the new Constraint object
	 */
	public Constraint createConstraint(Object data);
	
	/**
	 * The type of Constraints that this factory
	 * produces (e.g. ocl, prolog, etc.)
	 * @return the type of Constraint produced by
	 *         the factory
	 */
	public String getType();
}
