package org.eclipse.gmt.gems.model.intelligence;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/
public class EclipseKBFactory {
	
	public static final String TYPE_SYSTEM_EXTENSION_POINT = "org.eclipse.gmt.gems.model.intelligence.core.typesystem";
	public static final String EVALUATOR_EXTENSION_POINT = "org.eclipse.gmt.gems.model.intelligence.core.constraintevaluator";
	public static final String EVALUATOR_TYPE = "type";

	public static final String CLASS = "class";

	public static RefreshKB createKB(){
		RefreshKB kb = new RefreshKB();
		return configureKB(kb);
	}
	
	public static RefreshKB configureKB(RefreshKB kb) {
		
		try {
			List<Hashtable> extdat = new ArrayList<Hashtable>();
			Object[] constraintEvals = loadExtensions(
					EVALUATOR_EXTENSION_POINT, extdat);
			
			for(int i = 0; i < constraintEvals.length; i++){
				String type = (String)extdat.get(i).get(EVALUATOR_TYPE);
				ConstraintEvaluator e = (ConstraintEvaluator)constraintEvals[i];
				if(e instanceof RefreshKBAware){
					((RefreshKBAware)e).setRefreshKB(kb);
				}
				kb.getConstraintEvaluators().put(type, e);
				if(e instanceof KBManager){
					kb.getKnowledgeManagers().add((KBManager)e);
				}
			}
			
			Object[] typeSystems = loadExtensions(TYPE_SYSTEM_EXTENSION_POINT,
					new ArrayList<Hashtable>());
			for(Object o : typeSystems){
				kb.getTypeSystems().add((ObjectTypeSystem)o);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return kb;
	}
	
	public static void configureKBConstraintFactory(KBConstraintFactory fact){
		try {
			List<Hashtable> extdat = new ArrayList<Hashtable>();
			Object[] constraintfacts = loadExtensions(
					ConstraintFactory.EXTENSION_POINT, extdat);
			
			for(int i = 0; i < constraintfacts.length; i++){
				ConstraintFactory cfact = (ConstraintFactory)constraintfacts[i];
				fact.register(cfact);
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static Object[] loadExtensions(String extpoint,
			List<Hashtable> extdat) throws Exception {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		
		if(registry == null)
			return new Object[0];
		
		IExtensionPoint extensionPoint = registry.getExtensionPoint(extpoint);
		IExtension[] extensions = extensionPoint.getExtensions();

		extensions = extensionPoint.getExtensions();

		Vector result = new Vector();

		for (int i = 0; i < extensions.length; i++) {
			IExtension extension = extensions[i];
			IConfigurationElement[] elements = extension
					.getConfigurationElements();

			for (int j = 0; j < elements.length; j++) {
				IConfigurationElement element = elements[j];

				if (extdat != null) {

					Hashtable dat = new Hashtable();
					for (String attr : element.getAttributeNames()) {
						Object val = element.getAttribute(attr);
						if (val != null)
							dat.put(attr, val);
					}
					extdat.add(dat);

				}

				Object callback = element.createExecutableExtension(CLASS);
				Object as = callback;
				if (as != null) {
					result.add(as);
				}

			}
		}
		return result.toArray();
	}
}
