package org.eclipse.gmt.gems.model.intelligence;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/
public class KBFactory {

	private static KBFactory instance_;

	public static KBFactory getInstance() {
		if (instance_ == null) {
			instance_ = new KBFactory();
			instance_.getTypeSystems().add(new SimpleMapTypeSystem());
			instance_.addTypeSystem("org.refresh.emf.EObjectTypeSystem");
			instance_.addConstraintEvaluator("ocl", "org.eclipse.gmt.gems.model.intelligence.emf.OCLConstraintEvaluator");
			instance_.addConstraintEvaluator("groovy", "org.refresh.groovy.GroovyEvaluator");
			instance_.addConstraintEvaluator("bean-shell", "org.refresh.beanshell.BeanShellEvaluator");
		}
		return instance_;
	}

	private Map<String, ConstraintEvaluator> constraintEvaluators_ = new HashMap<String, ConstraintEvaluator>();
	private List<ObjectTypeSystem> typeSystems_ = new ArrayList<ObjectTypeSystem>();

	private KBFactory() {
	}

	public RefreshKB createKB() {
		RefreshKB kb = new RefreshKB();
		return configureKB(kb);
	}

	public RefreshKB configureKB(RefreshKB kb) {

		kb.getConstraintEvaluators().putAll(constraintEvaluators_);
		kb.getTypeSystems().addAll(typeSystems_);

		return kb;
	}

	public Map<String, ConstraintEvaluator> getConstraintEvaluators() {
		return constraintEvaluators_;
	}

	public void setConstraintEvaluators(
			Map<String, ConstraintEvaluator> constraintEvaluators) {
		constraintEvaluators_ = constraintEvaluators;
	}

	public List<ObjectTypeSystem> getTypeSystems() {
		return typeSystems_;
	}

	public void setTypeSystems(List<ObjectTypeSystem> typeSystems) {
		typeSystems_ = typeSystems;
	}

	public boolean addTypeSystem(String typesystemclass) {
		try {
			ObjectTypeSystem t = (ObjectTypeSystem) Class.forName(
					typesystemclass).newInstance();
			getTypeSystems().add(t);
		} catch (Throwable e) {
			return false;
		}
		return true;
	}
	
	public boolean addConstraintEvaluator(String type, String evalclass) {
		try {
			ConstraintEvaluator ce = (ConstraintEvaluator) Class.forName(
					evalclass).newInstance();
			getConstraintEvaluators().put(type,ce);
		} catch (Throwable e) {
			return false;
		}
		return true;
	}

}
