/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/
package org.eclipse.gmt.gems.model.intelligence;

public class ObjectTypes {

	/**
	 * This method returns true iff:
	 * <code>
	 *       typea v1 = ...;
	 *       typeb v2 = ...;
	 *       v1 = v2;
	 * </code>
	 * is legal. That is, typeb is either
	 * the same class as typea or is a subclass
	 * of typea.
	 * 
	 * @param a the reference type
	 * @param b the type that the "is a"
	 *          
	 * @return if b "is a" a
	 */
	public static boolean isAssignableFrom(ObjectType typea, ObjectType typeb){
		if(typeb == typea)
			return true;
		
		for(ObjectType d : typeb.getSuperTypes()){
			if(isAssignableFrom(typea, d))
				return true;
		}
		
		return false;
	}
}
