package org.eclipse.gmt.gems.model.intelligence;

import java.util.List;

/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and
 * the accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/

/**
 * This is an OPTIONAL interface for implementing a new type
 * of constraint language. Implementations of this interface
 * are responsible for taking a source object of
 * a relationship and a set of potential target objects
 * for the relationship and determining the subset of the
 * potential target objects that satisfy the constraint with
 * the given source object. Implementations of this interface
 * should be registered with the RefreshKB via a call to
 * <code>
 * QueryEvaluator eval = new MyQueryEvaluator();
 * RefreshKB kb = ...;
 * kb.getQueryEvaluators().add(eval);
 * </code>
 */
public interface QueryEvaluator {
	
	/**
	 * This method takes a source object, a set of constraints over
	 * a source and target object, and a set of potential 
	 * targets and returns the subset of the potential targets
	 * for which the constraints hold with the specified source.
	 * 
	 * The functionality of this method should be:
	 * <code>
	 * find all elements from world where each constraint in
	 * cons holds true if the source object is src.
	 * </code>
	 * 
	 * 
	 * @param src the source object
	 * @param world the list of potential target objects
	 * @param cons the constraints to satisfy
	 * @return the subset of world for which all cons hold true
	 */
	public List findAll(Object src, List world, List<Constraint> cons);
}
