/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/
package org.eclipse.gmt.gems.model.intelligence;

import java.util.regex.Pattern;


public class RegexKBMatcher<T> extends KBMatcher<T> {

	private class RegexFunction<T> implements Function<T,Boolean> {
		
		private Pattern pattern_;
		private Function<T, String> adapter_;

		public RegexFunction(String pattern, Function<T, String> adapter) {
			super();
			pattern_ = Pattern.compile(pattern);
			adapter_ = adapter;
		}

		
		public Boolean apply(T arg0) {
			return pattern_.matcher(adapter_.apply(arg0)).matches();
		}		
	}
	
	public class ToStringFunction<T> implements Function<T,String> {

		
		public String apply(T arg0) {
			return ""+arg0;
		}
		
	}
	
	private String patternSource_;
	
	public RegexKBMatcher(String pattern, Function<T, String> toStringConverter) {
		super();
		patternSource_ = pattern;
		setMatchingFunction(new RegexFunction<T>(pattern,toStringConverter));
	}
	
	public RegexKBMatcher(String pattern) {
		super();
		patternSource_ = pattern;
		setMatchingFunction(new RegexFunction<T>(pattern,new ToStringFunction<T>()));
	}

	
	public String toString(){
		return "RegexKBMatcher["+patternSource_+"]";
	}
}
